#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# => Module helper added to sensor helper. This module contains methods
#    to help the render of sensor's UI.
#
module RedBorder
  module SensorsHelperIPS
    def self.included(base)
      base.send(:include, InstanceMethods)

      base.class_eval do
        unloadable
        alias_method_chain :available_licenses, :ips_sensors

        #
        # => Return version depending on sensor's type
        #
        def ips_version(sensor_type)
          case sensor_type
          when Sensor::IPS_V4
            4
          when Sensor::IPS, Sensor::IPS_PROXY, Sensor::IPS_GENERIC
            3
          when Sensor::IPS_V2
            2
          end
        end

        def sensor_health_check_items_for_ips(sensor)
          items = []

          items << {
            check: I18n.t('sensors.healthcheck.license?'),
            status: sensor.license_id.present?,
            info: "#{I18n.t('sensors.healthcheck.expiration_date')}: #{sensor.license ? sensor.license.expire_at : 'N/A'}"
          }

          policies_present = sensor.policy_id.present? || sensor.reputation_policy_id.present?
          items << {
            check: I18n.t('sensors.healthcheck.sensor_policies?'),
            status: policies_present,
            info: "#{I18n.t('sensors.healthcheck.sensor_policies_signature')}: #{((sensor.policy && sensor.policy.rules) ? sensor.policy.rules.count : 0)} / #{I18n.t('sensors.healthcheck.sensor_policies_reputation')}: #{((sensor.reputation_policy && sensor.reputation_policy.reputations) ? sensor.reputation_policy.reputations.count : 0)}"
          }

          items << {
            check: I18n.t('sensors.healthcheck.signature_applied?'),
            status: !sensor.need_apply,
            info: I18n.t(sensor.need_apply ? 'sensors.healthcheck.signature_not' : 'sensors.healthcheck.signature_applied')
          }

          # items << {
          #   check: I18n.t('sensors.healthcheck.reputation_applied?'),
          #   status: !sensor.need_reputation_apply,
          #   info: I18n.t(sensor.need_reputation_apply ? 'sensors.healthcheck.reputation_not' : 'sensors.healthcheck.reputation_applied')
          # }

          if sensor.last_check_in
            segments = (sensor.chef_node && sensor.chef_node['redborder'] && sensor.chef_node['redborder']['segments']) || {}
            at_least_one_active = segments.any? { |_, segment| segment['bypass'] != true }
            items << {
              check: I18n.t('sensors.healthcheck.segment_bypass?'),
              status: at_least_one_active,
              info: I18n.t(at_least_one_active ? 'sensors.healthcheck.segment_bypass' : 'sensors.healthcheck.segment_bypass_not')
            }
          end
          items
        end
      end
    end

    module InstanceMethods
      #
      # Return a collection to be used in a select input that represent the available licenses
      # for a certain sensor.
      #
      def available_licenses_with_ips_sensors(sensor)
        type = (['sensor_proxy', 'sensor_v4'].include?(sensor.type_to_s)) ? 'sensor' : sensor.type_to_s
        licenses = License.available_by_type(type, sensor.ancestor_organization)
        licenses << sensor.license if sensor.license && !sensor.license.expired?

        licenses.uniq.map { |l| ["#{l.uuid} - #{l.expire_at.strftime('%Y-%m-%d')} (#{l.available_slots(type)})", l.id] }
      end
    end
  end
end
